;******************************************************************************
;                                                                             *
;    Filename:      LabSYSC05                                                 *
;    Date:          21.4.2010                                                 *
;    File Version:  6.0                                                       *
;                                                                             *
;    Author:        J.Sarboch   		                                      *
;    Company:       FEL CVUT                                                  *
;	 Descriptin:	Autonomni citac sudych hodnot							  *
;                                                                             * 
;******************************************************************************
;                                                                             *
;    Files Required: P18F87J11.INC                                            *
;                                                                             *
;******************************************************************************

	LIST P=18F87J11		;directive to define processor
	#include <P18F87J11.INC>	;processor specific variable definitions

;******************************************************************************
;Configuration bits
;Microchip has changed the format for defining the configuration bits, please 
;see the .inc file for futher details on notation.  Below are a few examples.



;   Oscillator Selection:
    CONFIG	FOSC = HS            ;HS oscilator
	CONFIG	WDTEN = OFF;         ;nepouzivame Watchdog
	CONFIG	XINST = OFF;		 ;nepouzivame rozsireny instrukcni soubor

;Preddefinovane konstanty

#define TMR0_PRELOAD_HIGH	0x67 ;pocatecni hodnota dec:26472
#define TMR0_PRELOAD_LOW	0x68+0x01

#define TMR1_PRELOAD_HIGH	0x9E ;pocatecni hodnota dec:40535
#define TMR1_PRELOAD_LOW	0x57+0x06

#define PROGRESS_STOP			0
#define PROGRESS_RUN			1


;******************************************************************************
;Variable definitions
; These variables are only needed if low priority interrupts are used. 
; More variables may be needed to store other special function registers used
; in the interrupt routines.

		CBLOCK	0x000
		WREG_TEMP				;variable used for context saving 
		STATUS_TEMP				;variable used for context saving
		BSR_TEMP				;variable used for context saving

		PROGRESS				;promenna udrzujici informaci o stavu (Run/Stop)
		COUNTERHIGH				;citac High priority preruseni
		COUNTERLOW				;citac Low priority preruseni
		COUNTER_ODD				;citac sudych hodnot High prio. preruseni
		STATE_BUTTONS			;stav citace a tlacitek XXSS XXBB (stavy 0-3)
		BUTTON_TOGGLE			;v simul. - tl. Toggle (zmena Run->Stop nebo Stop->Run)
		BUTTON_RESET			;v sumul. - tl. Reset (hodnota citace sudych stavu = 0)
		ENDC

;******************************************************************************
;Reset vector
; This code will start executing when a reset occurs.

		ORG	0x0000				;startovaci adresa po resetu

		goto	Main			;skok na hlavni cast programu

;******************************************************************************
;High priority interrupt vector
; This code will start executing when a high priority interrupt occurs or
; when any interrupt occurs if interrupt priorities are not enabled.

		ORG	0x0008				;vektor preruseni pro vysokou prioritu

		goto	HighInt			;skok na ISR pri preruseni s vysokou prioritou

;******************************************************************************
;Low priority interrupt vector and routine
; This code will start executing when a low priority interrupt occurs.
; This code can be removed if low priority interrupts are not used.

		ORG	0x0018					;vektor preruseni pro nizkou prioritu

		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register

		movlw	TMR1_PRELOAD_HIGH	;prednastavena konstanta (byte High) -> accum.
		movwf	TMR1H				;nastaveni pocatecni hodnoty (byte High) Timeru 1
		movlw	TMR1_PRELOAD_LOW	;prednastavena konstanta (byte Low) -> accum.
		movwf	TMR1L				;nastaveni pocatecni hodnoty (byte Low) Timeru 1
		incf	COUNTERLOW			;inkrementace citace Low prio. preruseni

		bcf		STATE_BUTTONS, 0, 0	;nastaveni xxxx_xxx0 (Reset uvolneno)
		btfss	PORTA, 5, 0			;Je RA5(Reset) stisknuto? (bit5=0 => stisknuto)
		bsf		STATE_BUTTONS, 0, 0	;nastaveni xxxx_xxx1 (Reset stisknuto)
	;	btfss	BUTTON_RESET, 0, 0	;Je BUTTON_RESET neaktivni? (nahrada RA5 v simul.)
	;	bsf		STATE_BUTTONS, 0, 0	;nastaveni xxxx_xxx1 (Reset stisknuto) v simul.

		bcf		STATE_BUTTONS, 1, 0	;nastaveni xxxx_xx0x (Toggle uvolneno)
		btfss	PORTB, 0, 0			;Je RB0(Toggle) stisknuto? (bit0=0 => stisknuto)
		bsf		STATE_BUTTONS, 1, 0	;nastaveni xxxx_xx1x (Toggle stisknuto)
	;	btfss	BUTTON_TOGGLE, 0, 0 ;Je BUTTON_TOGGLE neaktivni? (nahrada RB0 v simul.)
	;	bsf		STATE_BUTTONS, 1, 0	;nastaveni xxxx_xx1x (Toggle stisknuto) v simul.

		
		movlw	0x0F				;hodnota 0000 1111 -> maska nulovani LED 7,6 tlacitek
		andwf	PORTD, 1, 0			;nulovani LED 7,6 stavu tlacitek
		movff	STATE_BUTTONS, WREG	;XXSS_XXBB do accumulatoru
		mullw	0x40				;XXSS_XXBB -> BB00_0000 (nasobenim konst. a accum.)
		movff	PRODL, WREG			;BB00_0000 do accum.
		iorwf	PORTD, 1, 0			;aktualizace LED 7,6 tl.(log. soucet: 0 or aktual.)
		
		bcf		PIR1, TMR1IF, 0		;vynuluj priznak prave osetreneho preruseni TMR1

		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie						;navrat z ISR

;******************************************************************************
;High priority interrupt routine
; The high priority interrupt code is placed here to avoid conflicting with
; the low priority interrupt vector.

HighInt:
		movlw	TMR0_PRELOAD_HIGH	;prednastavena konstanta (byte High) -> accum.
		movwf	TMR0H				;nastaveni pocatecni hodnoty (byte High) Timeru 0
		movlw	TMR0_PRELOAD_LOW	;prednastavena konstanta (byte Low) -> accum.
		movwf	TMR0L				;nastaveni pocatecni hodnoty (byte Low) Timeru 0
		INCF 	COUNTERHIGH,1,0		;inkrementace citace High prio. preruseni
		movlw	PROGRESS_STOP		;PROGRESS_STOP do accum.
		cpfseq	PROGRESS			;PROGRESS = PROGRESS_STOP? (=ANO -> preskoc nasled.)
		bra		HighIntTimeRunning	;proved HighIntTimeRunning
		bra		HighIntEnd			;proved HighIntEnd

HighIntTimeRunning:		
		BTFSS 	COUNTERHIGH,0,0		;pokud citac High lichy, pak preskoc nasled. instr.
		INCF 	COUNTER_ODD,1,0		;inkrementace citace sudych hodnot High prio. prerus.
		movlw 	0x0F				;maska 000 1111 do accum.
		ANDWF 	COUNTER_ODD,1,0		;maska a citac -> rozsah 00 - 0F
		movff	COUNTER_ODD, PORTD	;zobraz aktualni hodnotu citace na LEDkach 3-0

HighIntEnd:		
		bcf		INTCON, TMR0IF, 0	;vynuluj priznak prave osetreneho preruseni
		retfie	FAST				;navrat z ISR s obnovenim kontextu

;******************************************************************************
;Start of main program
; The main program code is placed here.

Main:
		call 	Init				;proved proceduru Init

Loop:
		call	StateMachine
		goto	Loop				;nekonecna smycka

;******************************************************************************
;Start of Init routine

Init:	clrf	TRISD, 0			;konfiguruace PORTD jako vystup
		clrf	PORTD, 0			;nuluj hodnoty na PORTD (zadna LED nesviti)
		bsf		PROGRESS, 0, 0		;nastav PROGRESS bit0=1, Run=citani povoleno
		clrf	COUNTER_ODD			;nuluj citac sudych hodnot High Prio. prerus.
		clrf	COUNTERHIGH			;nuluj	citac High Prio. preruseni
		clrf	COUNTERLOW			;nuluj	citac Low Prio. preruseni

		clrf	STATE_BUTTONS		;nuluj stav citace a tlacitek, XX00_XX00
		
		bsf		BUTTON_TOGGLE, 0, 0	;nastaveni BUTTON_TOGGLE neaktivni (bit0=1)
		bsf		BUTTON_RESET, 0, 0	;nastaveni BUTTON_RESET neaktivni (bit0=1)

		movlw	b'00000001'			; hodnota b'00000001' do accum. (bit0=1)
		movwf	TRISB				;konfigurace PORTB jako vystup az na bit0 -> RB0

		movlw	b'00100000'			; hodnota b'00100000' do accum. (bit5=1)
		movwf	TRISA				;konfigurace PORTA jako vystup az na bit5 -> RA5
		bsf 	WDTCON, ADSHR		;!!! Enable write/read to the shared SFR	!!!
		bsf		ANCON0, PCFG4		;!!! Set RA5=digital port enable, else AN4!	!!!
		bcf		WDTCON, ADSHR		;!!! Disable write/read to the shared SFR	!!!

		movlw	b'10000000'			;konfig. preruseni -> hodnota b'10000000' do accum.
		movwf	RCON				;povoleni priorit u preruseni

		movlw	b'00000101'			;konfig. Timer0 -> hodnota b'00000101' do accum.
		movwf	T0CON				;Timer0 ma 16b, z internich hodin, s 1:64 prescalerem
		movlw	b'11100000'			;konfig. Timer0 -> hodnota b'11100000' do accum.
		movwf	INTCON				;preruseni od preteceni Timer0 povoleno
		movlw	b'00000100'			;konfig. Timer0 -> hodnota b'00000100' do accum.
		movwf	INTCON2				;preruseni od Timer0 ma vysokou prioritu

		movlw	b'10000000'			;konfig. Timer1 -> hodnota b'10000000' do accum.		
		movwf	T1CON				;Timer1 ma 16b, z internich hodin, s 1:1 prescalerem
		bcf 	IPR1, TMR1IP, 0		;preruseni od Timer1 ma nizkou prioritu
		bsf		PIE1, TMR1IE, 0		;povol preruseni od Timer1

		movlw	TMR0_PRELOAD_HIGH	;prednastavena konstanta (byte High) -> accum.
		movwf	TMR0H				;nastaveni pocatecni hodnoty (byte High) Timeru 0
		movlw	TMR0_PRELOAD_LOW	;prednastavena konstanta (byte Low) -> accum.
		movwf	TMR0L				;nastaveni pocatecni hodnoty (byte Low) Timeru 0
		movlw	TMR1_PRELOAD_HIGH	;prednastavena konstanta (byte High) -> accum.
		movwf	TMR1H				;nastaveni pocatecni hodnoty (byte High) Timeru 1
		movlw	TMR1_PRELOAD_LOW	;prednastavena konstanta (byte Low) -> accum.
		movwf	TMR1L				;nastaveni pocatecni hodnoty (byte Low) Timeru 1

		bsf		T1CON, TMR1ON, 0 	;spust Timer1
		bsf		T0CON, TMR0ON, 0 	;spust Timer0

		return

;******************************************************************************
;Start of StateMachine routine

StateMachine:
		bra		St0Bu0				;jdi na navesti St0Bu0 (STATE_BUTTONS:XX00_XX00)

St0Bu0:	movlw	0x00
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX00_XX00 -> preskoc nasl. instr.
		bra 	St0Bu1				;jdi na navesti St0Bu1 (STATE_BUTTONS:XX00_XX01)
		bcf		STATE_BUTTONS, 4, 0	;nastav XXX0_XXBB
		bcf		STATE_BUTTONS, 5, 0	;nastav XX00_XXBB

St0Bu1:	movlw	0x01
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX00_XX01 -> preskoc nasl. instr.
		bra 	St0Bu2				;jdi na navesti St0Bu2 (STATE_BUTTONS:XX00_XX10)
		clrf	COUNTER_ODD			;Reset citace sudych High prio. preruseni
		bsf		STATE_BUTTONS, 4, 0	;nastav XXX1_XXBB
		bcf		STATE_BUTTONS, 5, 0	;nastav XX01_XXBB

St0Bu2:	movlw	0x02
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX00_XX10 -> preskoc nasl. instr.
		bra		St0Bu3				;jdi na navesti St0Bu3 (STATE_BUTTONS:XX00_XX11)
		btg		PROGRESS, 0, 0		;Toggle stav Run<->Stop
		bcf		STATE_BUTTONS, 4, 0	;nastav XXX0_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX10_XXBB

St0Bu3:	movlw	0x03
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX00_XX11 -> preskoc nasl. instr.
		bra		St1Bu0				;jdi na navesti St1Bu0 (STATE_BUTTONS:XX01_XX00)
		btg		PROGRESS, 0, 0		;Toggle stav Run<->Stop
		clrf	COUNTER_ODD			;Reset citace sudych High prio. preruseni
		bsf		STATE_BUTTONS, 4, 0	;nastav XXX1_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX11_XXBB

St1Bu0:	movlw	0x10
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX01_XX00 -> preskoc nasl. instr.
		bra		St1Bu1				;jdi na navesti St1Bu1 (STATE_BUTTONS:XX01_XX01)
		bcf		STATE_BUTTONS, 4, 0	;nastav XXX0_XXBB
		bcf		STATE_BUTTONS, 5, 0	;nastav XX00_XXBB

St1Bu1:	movlw	0x11				
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX01_XX01 -> preskoc nasl. instr.
		bra 	St1Bu2				;jdi na navesti St1Bu2 (STATE_BUTTONS:XX01_XX10)
		bsf		STATE_BUTTONS, 4, 0	;nastav XXX1_XXBB
		bcf		STATE_BUTTONS, 5, 0	;nastav XX01_XXBB

St1Bu2:	movlw	0x12
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX01_XX10 -> preskoc nasl. instr.
		bra		St1Bu3				;jdi na navesti St1Bu3 (STATE_BUTTONS:XX01_XX11)
		btg		PROGRESS, 0, 0		;Toggle stav Run<->Stop
		bcf		STATE_BUTTONS, 4, 0	;nastav XXX0_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX10_XXBB

St1Bu3:	movlw	0x13
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX01_XX11 -> preskoc nasl. instr.
		bra 	St2Bu0				;jdi na navesti St2Bu0 (STATE_BUTTONS:XX10_XX00)
		btg		PROGRESS, 0, 0		;Toggle stav Run<->Stop
		bsf		STATE_BUTTONS, 4, 0	;nastav XXX1_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX11_XXBB
		
St2Bu0:	movlw	0x20
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX10_XX00 -> preskoc nasl. instr.
		bra		St2Bu1				;jdi na navesti St2Bu1 (STATE_BUTTONS:XX10_XX01)
		bcf		STATE_BUTTONS, 4, 0	;nastav XXX0_XXBB
		bcf		STATE_BUTTONS, 5, 0	;nastav XX00_XXBB

St2Bu1:	movlw	0x21
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX10_XX01 -> preskoc nasl. instr.
		bra		St2Bu2				;jdi na navesti St2Bu2 (STATE_BUTTONS:XX10_XX10)
		clrf 	COUNTER_ODD			;Reset citace sudych High prio. preruseni
		bsf		STATE_BUTTONS, 4, 0	;nastav XXX1_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX11_XXBB

St2Bu2:	movlw	0x22
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX10_XX10 -> preskoc nasl. instr.
		bra		St2Bu3				;jdi na navesti St2Bu3 (STATE_BUTTONS:XX10_XX11)
		bcf		STATE_BUTTONS, 4, 0	;nastav XXX0_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX10_XXBB

St2Bu3:	movlw	0x23
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX10_XX11 -> preskoc nasl. instr.
		bra 	St3Bu0				;jdi na navesti St3Bu0 (STATE_BUTTONS:XX11_XX00)
		clrf	COUNTER_ODD			;Reset citace sudych High prio. preruseni
		bsf		STATE_BUTTONS, 4, 0	;nastav XXX1_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX11_XXBB

St3Bu0:	movlw	0x30
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX11_XX00 -> preskoc nasl. instr.
		bra		St3Bu1				;jdi na navesti St3Bu1 (STATE_BUTTONS:XX11_XX01)
		bcf		STATE_BUTTONS, 4, 0	;nastav XXX0_XXBB
		bcf		STATE_BUTTONS, 5, 0	;nastav XX00_XXBB

St3Bu1:	movlw	0x31
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX11_XX01 -> preskoc nasl. instr.
		bra		St3Bu2				;jdi na navesti St3Bu2 (STATE_BUTTONS:XX11_XX10)
		bsf		STATE_BUTTONS, 4, 0	;nastav XXX1_XXBB
		bcf		STATE_BUTTONS, 5, 0	;nastav XX01_XXBB

St3Bu2:	movlw	0x32
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX11_XX10 -> preskoc nasl. instr.
		bra 	St3Bu3				;jdi na navesti St3Bu3 (STATE_BUTTONS:XX11_XX11)
		bcf		STATE_BUTTONS, 4, 0	;nastav XXX0_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX10_XXBB

St3Bu3:	movlw	0x33
		cpfseq	STATE_BUTTONS		;je-li XXSS_XXBB=XX11_XX11 -> preskoc nasl. instr.
		bra		Leave				;jdi na navesti Leave (aktualizace LED 3-0)
		bsf		STATE_BUTTONS, 4, 0	;nastav XXX1_XXBB
		bsf		STATE_BUTTONS, 5, 0	;nastav XX11_XXBB

Leave:	movff	PORTD, WREG			;presun hodnotu PORTD do accum.
		andlw	0x0F				;vynuluj na b'0000XXXX'
		cpfseq	COUNTER_ODD			; shoda sudeho citace a accum? preskoc nasl. instr.
		bra		UpDCnt				;jdi na aktualizaci LEDek 3-0 PORTD
		return

UpDCnt:	movlw	0xC0				;hodnota b'11000000' do accum.
		andwf	PORTD, 1, 0			;nastav PORTD na b'XX000000'
		movff	COUNTER_ODD,WREG	;presun COUNTER_ODD do accum.
		iorwf	PORTD, 1, 0			; Proved aktualizaci LEDek 3-0 PORTD dle COUNTER_ODD
		return

		END