;******************************************************************************
;                                                                             *
;    Filename:      Uloha 1                                                   *
;    Date:          29.2.2010                                                 *
;    File Version:  1.0                                                       *
;                                                                             *
;    Author:        Miroslav Chomat                                           *
;    Company:       FEL CVUT                                                  *
;                                                                             * 
;******************************************************************************
;                                                                             *
;    Files Required: P18F87J11.INC                                            *
;                                                                             *
;******************************************************************************

	LIST P=18F87J11		;directive to define processor
	#include <P18F87J11.INC>	;processor specific variable definitions

;******************************************************************************
;Configuration bits
;Microchip has changed the format for defining the configuration bits, please 
;see the .inc file for futher details on notation.  Below are a few examples.



;   Oscillator Selection:
    CONFIG	FOSC = HS            ;HS oscilator
	CONFIG	WDTEN = OFF;         ;nepouzivame Watchdog
	CONFIG	XINST = OFF;		 ;nepouzivame rozsireny instrukcni soubor


;******************************************************************************
;Variable definitions
; These variables are only needed if low priority interrupts are used. 
; More variables may be needed to store other special function registers used
; in the interrupt routines.

		REG EQU 0x000			;nase promenna

;******************************************************************************
;Reset vector
; This code will start executing when a reset occurs.

		ORG	0x0000			;startovaci adresa po resetu

		goto	Main		;skok na hlavni cast programu

;******************************************************************************
;High priority interrupt vector
; This code will start executing when a high priority interrupt occurs or
; when any interrupt occurs if interrupt priorities are not enabled.

		ORG	0x0008			;vektor preruseni pro vysokou prioritu

		goto	HighInt		;skok na ISR pri preruseni s vysokou prioritou

;******************************************************************************
;Low priority interrupt vector and routine
; This code will start executing when a low priority interrupt occurs.
; This code can be removed if low priority interrupts are not used.

		ORG	0x0018			;vektor preruseni pro nizkou prioritu

		retfie				;navrat z ISR

;******************************************************************************
;High priority interrupt routine
; The high priority interrupt code is placed here to avoid conflicting with
; the low priority interrupt vector.

HighInt:
		rlncf	REG, 1, 0			;Rotace vlevo 
		btfss	PORTB, 0, 0			;Je RB0 stisknuto?
		bsf		REG, 0, 0			;Nastav bit0 na REG na 1
		movff	REG, PORTD			;REG -> PORTD
		bcf		INTCON, 2, 0		;vynuluj priznak osetreneho preruseni
		retfie	FAST				;navrat z ISR s obnovenim kontextu

;******************************************************************************
;Start of main program
; The main program code is placed here.

Main:
		call 	Init

Loop:
		goto Loop				;nekonecna smycka


Init:	clrf	TRISD, 0		;Konfiguruace PORTD jako vystup
		clrf	PORTD, 0		;Inicializace hodnot na PORTD
		clrf	REG, 0			;Inicializace nasi promenne

		movlw	b'00000001'		;Konfigurace PORTB jako vystup az na bit0
		movwf	TRISB

		movlw	b'00000010'		;Timer0 ma 16b, z internich hodin, s 1:8 prescalerem
		movwf	T0CON

		movlw	b'10100000'		;Preruseni od preteceni Timer0 povolen
		movwf	INTCON

		movlw	b'00000100'		;Preruseni od Timer0 ma vysokou prioritu
		movwf	INTCON2

		movlw	b'10000000'		;Povoleni priorit u preruseni
		movwf	RCON

		bsf		T0CON, 7, 0		;Spust Timer0

		return


		END
